@@ -28,7 +28,7 @@ from .utils import (
 logger = logging.getLogger("fsspec")
 
 
-class OpenFile(object):
+class OpenFile:
     """
     File-like object to be used in a context
 
@@ -36,7 +36,7 @@ class OpenFile(object):
     are typically binary-only.
 
     These instances are safe to serialize, as the low-level file object
-    is not created until invoked using `with`.
+    is not created until invoked using ``with``.
 
     Parameters
     ----------
@@ -55,6 +55,10 @@ class OpenFile(object):
         How to handle encoding errors if opened in text mode.
     newline: None or str
         Passed to TextIOWrapper in text mode, how to handle line endings.
+    autoopen: bool
+        If True, calls open() immediately. Mostly used by pickle
+    pos: int
+        If given and autoopen is True, seek to this location immediately
     """
 
     def __init__(
@@ -93,10 +97,6 @@ class OpenFile(object):
     def __repr__(self):
         return "<OpenFile '{}'>".format(self.path)
 
-    def __fspath__(self):
-        # may raise if cannot be resolved to local file
-        return self.open().__fspath__()
-
     def __enter__(self):
         mode = self.mode.replace("t", "").replace("b", "") + "b"
 
@@ -111,7 +111,7 @@ class OpenFile(object):
 
         if "b" not in self.mode:
             # assume, for example, that 'r' is equivalent to 'rt' as in builtin
-            f = io.TextIOWrapper(
+            f = PickleableTextIOWrapper(
                 f, encoding=self.encoding, errors=self.errors, newline=self.newline
             )
             self.fobjects.append(f)
@@ -121,10 +121,6 @@ class OpenFile(object):
     def __exit__(self, *args):
         self.close()
 
-    def __del__(self):
-        if hasattr(self, "fobjects"):
-            self.fobjects.clear()  # may cause cleanup of objects and close files
-
     @property
     def full_name(self):
         return _unstrip_protocol(self.path, self.fs)
@@ -132,27 +128,19 @@ class OpenFile(object):
     def open(self):
         """Materialise this as a real open file without context
 
-        The file should be explicitly closed to avoid enclosed file
-        instances persisting. This code-path monkey-patches the file-like
-        objects, so they can close even if the parent OpenFile object has already
-        been deleted; but a with-context is better style.
+        The OpenFile object should be explicitly closed to avoid enclosed file
+        instances persisting. You must, therefore, keep a reference to the OpenFile
+        during the life of the file-like it generates.
         """
-        out = self.__enter__()
-        closer = out.close
-        fobjects = self.fobjects.copy()[:-1]
-        mode = self.mode
-
-        def close():
-            # this func has no reference to
-            closer()  # original close bound method of the final file-like
-            _close(fobjects, mode)  # call close on other dependent file-likes
-
-        out.close = close
-        return out
+        return self.__enter__()
 
     def close(self):
         """Close all encapsulated file objects"""
-        _close(self.fobjects, self.mode)
+        for f in reversed(self.fobjects):
+            if "r" not in self.mode and not f.closed:
+                f.flush()
+            f.close()
+        self.fobjects.clear()
 
 
 class OpenFiles(list):
@@ -191,31 +179,28 @@ class OpenFiles(list):
 
     def __exit__(self, *args):
         fs = self.fs
+        [s.__exit__(*args) for s in self]
         if "r" not in self.mode:
             while True:
                 if hasattr(fs, "open_many"):
                     # check for concurrent cache upload
                     fs.commit_many(self.files)
-                    self.files.clear()
                     return
                 if hasattr(fs, "fs") and fs.fs is not None:
                     fs = fs.fs
                 else:
                     break
-        [s.__exit__(*args) for s in self]
+
+    def __getitem__(self, item):
+        out = super().__getitem__(item)
+        if isinstance(item, slice):
+            return OpenFiles(out, mode=self.mode, fs=self.fs)
+        return out
 
     def __repr__(self):
         return "<List of %s OpenFile instances>" % len(self)
 
 
-def _close(fobjects, mode):
-    for f in reversed(fobjects):
-        if "r" not in mode and not f.closed:
-            f.flush()
-        f.close()
-    fobjects.clear()
-
-
 def open_files(
     urlpath,
     mode="rb",
@@ -522,7 +507,8 @@ def strip_protocol(urlpath):
 
 
 def expand_paths_if_needed(paths, mode, num, fs, name_function):
-    """Expand paths if they have a ``*`` in them.
+    """Expand paths if they have a ``*`` in them (write mode) or any of ``*?[]``
+    in them (read mode).
 
     :param paths: list of paths
     mode: str
@@ -538,23 +524,32 @@ def expand_paths_if_needed(paths, mode, num, fs, name_function):
     """
     expanded_paths = []
     paths = list(paths)
-    if "w" in mode and sum([1 for p in paths if "*" in p]) > 1:
-        raise ValueError("When writing data, only one filename mask can be specified.")
-    elif "w" in mode:
+
+    if "w" in mode:  # read mode
+        if sum([1 for p in paths if "*" in p]) > 1:
+            raise ValueError(
+                "When writing data, only one filename mask can be specified."
+            )
         num = max(num, len(paths))
-    for curr_path in paths:
-        if "*" in curr_path:
-            if "w" in mode:
+
+        for curr_path in paths:
+            if "*" in curr_path:
                 # expand using name_function
                 expanded_paths.extend(_expand_paths(curr_path, name_function, num))
             else:
+                expanded_paths.append(curr_path)
+        # if we generated more paths that asked for, trim the list
+        if len(expanded_paths) > num:
+            expanded_paths = expanded_paths[:num]
+
+    else:  # read mode
+        for curr_path in paths:
+            if has_magic(curr_path):
                 # expand using glob
                 expanded_paths.extend(fs.glob(curr_path))
-        else:
-            expanded_paths.append(curr_path)
-    # if we generated more paths that asked for, trim the list
-    if "w" in mode and len(expanded_paths) > num:
-        expanded_paths = expanded_paths[:num]
+            else:
+                expanded_paths.append(curr_path)
+
     return expanded_paths
 
 
@@ -684,3 +679,26 @@ def _expand_paths(path, name_function, num):
             "3. A path with a '*' in it: 'foo.*.json'"
         )
     return paths
+
+
+class PickleableTextIOWrapper(io.TextIOWrapper):
+    """TextIOWrapper cannot be pickled. This solves it.
+
+    Requires that ``buffer`` be pickleable, which all instances of
+    AbstractBufferedFile are.
+    """
+
+    def __init__(
+        self,
+        buffer,
+        encoding=None,
+        errors=None,
+        newline=None,
+        line_buffering=False,
+        write_through=False,
+    ):
+        self.args = buffer, encoding, errors, newline, line_buffering, write_through
+        super().__init__(*self.args)
+
+    def __reduce__(self):
+        return PickleableTextIOWrapper, self.args
