@@ -44,11 +44,11 @@ class FUSEr(Operations):
         else:
             data["st_mode"] = stat.S_IFREG | perm
             data["st_size"] = info["size"]
-            data["st_blksize"] = 5 * 2 ** 20
+            data["st_blksize"] = 5 * 2**20
             data["st_nlink"] = 1
-        data["st_atime"] = time.time()
-        data["st_ctime"] = time.time()
-        data["st_mtime"] = time.time()
+        data["st_atime"] = info["atime"] if "atime" in info else time.time()
+        data["st_ctime"] = info["ctime"] if "ctime" in info else time.time()
+        data["st_mtime"] = info["mtime"] if "mtime" in info else time.time()
         return data
 
     def readdir(self, path, fh):
@@ -82,6 +82,7 @@ class FUSEr(Operations):
     def write(self, path, data, offset, fh):
         logger.debug("write %s", (path, offset))
         f = self.cache[fh]
+        f.seek(offset)
         f.write(data)
         return len(data)
 
@@ -175,8 +176,8 @@ def run(
         within the mounter directory. Operation will typically be more
         stable if False.
     ready_file: bool
-        Whether the FUSE process is ready. The `.fuse_ready` file will
-        exist in the `mount_point` directory if True. Debugging purpose.
+        Whether the FUSE process is ready. The ``.fuse_ready`` file will
+        exist in the ``mount_point`` directory if True. Debugging purpose.
     ops_class: FUSEr or Subclass of FUSEr
         To override the default behavior of FUSEr. For Example, logging
         to file.
