@@ -14,13 +14,33 @@ except ImportError:
 
 from ..asyn import AsyncFileSystem, sync
 from ..callbacks import _DEFAULT_CALLBACK
-from ..core import filesystem, open
-from ..mapping import get_mapper
+from ..core import filesystem, open, split_protocol
 from ..spec import AbstractFileSystem
+from ..utils import isfilelike, merge_offset_ranges
 
 logger = logging.getLogger("fsspec.reference")
 
 
+def _first(d):
+    return list(d.values())[0]
+
+
+def _prot_in_references(path, references):
+    ref = references.get(path)
+    if isinstance(ref, (list, tuple)):
+        return split_protocol(ref[0])[0] if ref[0] else ref[0]
+
+
+def _protocol_groups(paths, references):
+    if isinstance(paths, str):
+        return {_prot_in_references(paths, references): [paths]}
+    out = {}
+    for path in paths:
+        protocol = _prot_in_references(path, references)
+        out.setdefault(protocol, []).append(path)
+    return out
+
+
 class ReferenceFileSystem(AsyncFileSystem):
     """View byte ranges of some other file as a file system
 
@@ -56,8 +76,9 @@ class ReferenceFileSystem(AsyncFileSystem):
         fs=None,
         template_overrides=None,
         simple_templates=True,
+        max_gap=64_000,
+        max_block=256_000_000,
         loop=None,
-        ref_type=None,
         **kwargs,
     ):
         """
@@ -66,18 +87,19 @@ class ReferenceFileSystem(AsyncFileSystem):
         ----------
         fo : dict or str
             The set of references to use for this instance, with a structure as above.
-            If str, will use fsspec.open, in conjunction with ref_storage_args to
-            open and parse JSON at this location.
+            If str, will use fsspec.open, in conjunction with target_options
+            and target_protocol to open and parse JSON at this location.
         target : str
             For any references having target_url as None, this is the default file
             target to use
         ref_storage_args : dict
-            If references is a str, use these kwargs for loading the JSON file
+            If references is a str, use these kwargs for loading the JSON file.
+            Deprecated: use target_options instead.
         target_protocol : str
             Used for loading the reference file, if it is a path. If None, protocol
             will be derived from the given path
         target_options : dict
-            Extra FS options for loading the reference file, if given as a path
+            Extra FS options for loading the reference file ``fo``, if given as a path
         remote_protocol : str
             The protocol of the filesystem on which the references will be evaluated
             (unless fs is provided). If not given, will be derived from the first
@@ -85,19 +107,29 @@ class ReferenceFileSystem(AsyncFileSystem):
             order.
         remote_options : dict
             kwargs to go with remote_protocol
-        fs : file system instance
-            Directly provide a file system, if you want to configure it beforehand. This
-            takes precedence over target_protocol/target_options
+        fs : AbstractFileSystem | dict(str, (AbstractFileSystem | dict))
+            Directly provide a file system(s):
+                - a single filesystem instance
+                - a dict of protocol:filesystem, where each value is either a filesystem
+                  instance, or a dict of kwargs that can be used to create in
+                  instance for the given protocol
+
+            If this is given, remote_options and remote_protocol are ignored.
         template_overrides : dict
             Swap out any templates in the references file with these - useful for
             testing.
-        ref_type : "json" | "parquet" | "zarr"
-            If None, guessed from URL suffix, defaulting to JSON. Ignored if fo
-            is not a string.
         simple_templates: bool
             Whether templates can be processed with simple replace (True) or if
             jinja  is needed (False, much slower). All reference sets produced by
             ``kerchunk`` are simple in this sense, but the spec allows for complex.
+        max_gap, max_block: int
+            For merging multiple concurrent requests to the same remote file.
+            Neighboring byte ranges will only be merged when their
+            inter-range gap is <= ``max_gap``. Default is 64KB. Set to 0
+            to only merge when it requires no extra bytes. Pass a negative
+            number to disable merging, appropriate for local target files.
+            Neighboring byte ranges will only be merged when the size of
+            the aggregated range is <= ``max_block``. Default is 256MB.
         kwargs : passed to parent class
         """
         super().__init__(loop=loop, **kwargs)
@@ -106,6 +138,9 @@ class ReferenceFileSystem(AsyncFileSystem):
         self.template_overrides = template_overrides
         self.simple_templates = simple_templates
         self.templates = {}
+        self.fss = {}
+        self.max_gap = max_gap
+        self.max_block = max_block
         if hasattr(fo, "read"):
             text = fo.read()
         elif isinstance(fo, str):
@@ -114,82 +149,71 @@ class ReferenceFileSystem(AsyncFileSystem):
             else:
                 extra = {}
             dic = dict(**(ref_storage_args or target_options or {}), **extra)
-            if ref_type == "zarr" or fo.endswith("zarr"):
-                import pandas as pd
-                import zarr
-
-                self.dataframe = True
-                m = get_mapper(fo, **dic)
-                z = zarr.open_group(m)
-                assert z.attrs["version"] == 1
-                self.templates = z.attrs["templates"]
-                self.gen = z.attrs.get("gen", None)
-                self.df = pd.DataFrame(
-                    {k: z[k][:] for k in ["key", "data", "url", "offset", "size"]}
-                ).set_index("key")
-            elif ref_type == "parquet" or fo.endswith("parquet"):
-                import fastparquet as fp
-
-                self.dataframe = True
-                with open(fo, "rb", **dic) as f:
-                    pf = fp.ParquetFile(f)
-                    assert pf.key_value_metadata["version"] == 1
-                    self.templates = json.loads(pf.key_value_metadata["templates"])
-                    self.gen = json.loads(pf.key_value_metadata.get("gen", "[]"))
-                    self.df = pf.to_pandas(index="key")
-            else:
-                # text JSON
-                with open(fo, "rb", **dic) as f:
-                    logger.info("Read reference from URL %s", fo)
-                    text = f.read()
+            # text JSON
+            with open(fo, "rb", **dic) as f:
+                logger.info("Read reference from URL %s", fo)
+                text = f.read()
         else:
-            # dictionaries; TODO: allow dataframe here?
+            # dictionaries
             text = fo
         if self.dataframe:
             self._process_dataframe()
         else:
             self._process_references(text, template_overrides)
-        if fs is not None:
-            self.fs = fs
+        if isinstance(fs, dict):
+            self.fss = {
+                k: (
+                    fsspec.filesystem(k.split(":", 1)[0], **opts)
+                    if isinstance(opts, dict)
+                    else opts
+                )
+                for k, opts in fs.items()
+            }
+            if None not in self.fss:
+                self.fss[None] = filesystem("file")
             return
+        if fs is not None:
+            # single remote FS
+            remote_protocol = (
+                fs.protocol[0] if isinstance(fs.protocol, tuple) else fs.protocol
+            )
+            self.fss[remote_protocol] = fs
+
         if remote_protocol is None:
+            # get single protocol from any templates
             for ref in self.templates.values():
                 if callable(ref):
                     ref = ref()
                 protocol, _ = fsspec.core.split_protocol(ref)
-                if protocol:
-                    remote_protocol = protocol
-                    break
+                if protocol and protocol not in self.fss:
+                    fs = filesystem(protocol, loop=loop, **(remote_options or {}))
+                    self.fss[protocol] = fs
         if remote_protocol is None:
+            # get single protocol from references
             for ref in self.references.values():
                 if callable(ref):
                     ref = ref()
                 if isinstance(ref, list) and ref[0]:
                     protocol, _ = fsspec.core.split_protocol(ref[0])
-                    if protocol:
-                        remote_protocol = protocol
-                        break
-        if remote_protocol is None:
-            remote_protocol = target_protocol
+                    if protocol and protocol not in self.fss:
+                        fs = filesystem(protocol, loop=loop, **(remote_options or {}))
+                        self.fss[protocol] = fs
 
-        self.fs = filesystem(remote_protocol, loop=loop, **(remote_options or {}))
+        if remote_protocol and remote_protocol not in self.fss:
+            fs = filesystem(remote_protocol, loop=loop, **(remote_options or {}))
+            self.fss[remote_protocol] = fs
+
+        self.fss[None] = fs or filesystem("file")  # default one
 
     @property
     def loop(self):
-        return self.fs.loop if self.fs.async_impl else self._loop
+        inloop = [fs.loop for fs in self.fss.values() if fs.async_impl]
+        return inloop[0] if inloop else self._loop
 
-    def _cat_common(self, path):
+    def _cat_common(self, path, start=None, end=None):
         path = self._strip_protocol(path)
         logger.debug(f"cat: {path}")
-        # TODO: can extract and cache templating here
-        if self.dataframe:
-            part = self.df.loc[path]
-            if part["data"]:
-                part = part["data"]
-            else:
-                part = part[["url", "offset", "size"]]
-        else:
-            part = self.references[path]
+        part = self.references[path]
         if isinstance(part, str):
             part = part.encode()
         if isinstance(part, bytes):
@@ -201,27 +225,44 @@ class ReferenceFileSystem(AsyncFileSystem):
         if len(part) == 1:
             logger.debug(f"Reference: {path}, whole file")
             url = part[0]
-            start = None
-            end = None
+            start1, end1 = start, end
         else:
-            url, start, size = part
-            logger.debug(f"Reference: {path}, offset {start}, size {size}")
-            end = start + size
+            url, start0, size = part
+            logger.debug(f"Reference: {path} => {url}, offset {start0}, size {size}")
+            end0 = start0 + size
+
+            if start is not None:
+                if start >= 0:
+                    start1 = start0 + start
+                else:
+                    start1 = end0 + start
+            else:
+                start1 = start0
+            if end is not None:
+                if end >= 0:
+                    end1 = start0 + end
+                else:
+                    end1 = end0 + end
+            else:
+                end1 = end0
         if url is None:
             url = self.target
-        return url, start, end
+        return url, start1, end1
 
     async def _cat_file(self, path, start=None, end=None, **kwargs):
-        part_or_url, start0, end0 = self._cat_common(path)
+        part_or_url, start0, end0 = self._cat_common(path, start=start, end=end)
         if isinstance(part_or_url, bytes):
             return part_or_url[start:end]
-        return (await self.fs._cat_file(part_or_url, start=start0, end=end0))[start:end]
+        protocol, _ = split_protocol(part_or_url)
+        return await self.fss[protocol]._cat_file(part_or_url, start=start, end=end)
 
     def cat_file(self, path, start=None, end=None, **kwargs):
-        part_or_url, start0, end0 = self._cat_common(path)
+        part_or_url, start0, end0 = self._cat_common(path, start=start, end=end)
         if isinstance(part_or_url, bytes):
             return part_or_url[start:end]
-        return self.fs.cat_file(part_or_url, start=start0, end=end0)[start:end]
+        protocol, _ = split_protocol(part_or_url)
+        # TODO: start and end should be passed to cat_file, not sliced
+        return self.fss[protocol].cat_file(part_or_url, start=start0, end=end0)
 
     def pipe_file(self, path, value, **_):
         """Temporarily add binary data or reference as a file"""
@@ -238,33 +279,96 @@ class ReferenceFileSystem(AsyncFileSystem):
         if self.isdir(rpath):
             return os.makedirs(lpath, exist_ok=True)
         data = self.cat_file(rpath, **kwargs)
-        callback.lazy_call("set_size", len, data)
-        with open(lpath, "wb") as f:
-            f.write(data)
-        callback.lazy_call("absolute_update", len, data)
+        callback.set_size(len(data))
+        if isfilelike(lpath):
+            lpath.write(data)
+        else:
+            with open(lpath, "wb") as f:
+                f.write(data)
+        callback.absolute_update(len(data))
 
     def get(self, rpath, lpath, recursive=False, **kwargs):
-        if self.fs.async_impl:
-            return sync(self.loop, self._get, rpath, lpath, recursive, **kwargs)
-        return AbstractFileSystem.get(self, rpath, lpath, recursive=recursive, **kwargs)
+        if isinstance(lpath, list):
+            # because we have to figure out here which lpath goes with which path
+            # after grouping
+            raise NotImplementedError
+        proto_dict = _protocol_groups(rpath, self.references)
+        for proto, paths in proto_dict.items():
+            if self.fss[proto].async_impl:
+                sync(self.loop, self._get, paths, lpath, recursive, **kwargs)
+            else:
+                AbstractFileSystem.get(
+                    self, paths, lpath, recursive=recursive, **kwargs
+                )
 
-    def cat(self, path, recursive=False, **kwargs):
-        if self.fs.async_impl:
-            return sync(self.loop, self._cat, path, recursive, **kwargs)
-        elif isinstance(path, list):
-            if recursive or any("*" in p for p in path):
-                raise NotImplementedError
-            return {p: AbstractFileSystem.cat_file(self, p, **kwargs) for p in path}
-        else:
-            return AbstractFileSystem.cat_file(self, path)
+    def cat(self, path, recursive=False, on_error="raise", **kwargs):
+        if isinstance(path, str) and recursive:
+            raise NotImplementedError
+        if isinstance(path, list) and (recursive or any("*" in p for p in path)):
+            raise NotImplementedError
+        proto_dict = _protocol_groups(path, self.references)
+        out = {}
+        for proto, paths in proto_dict.items():
+            fs = self.fss[proto]
+            urls, starts, ends = zip(*[self._cat_common(p) for p in paths])
+            urls2 = []
+            starts2 = []
+            ends2 = []
+            paths2 = []
+            whole_files = set()
+            for u, s, e, p in zip(urls, starts, ends, paths):
+                if isinstance(u, bytes):
+                    # data
+                    out[p] = u
+                elif s is None:
+                    # whole file - limits are None, None, but no further
+                    # entries take for this file
+                    whole_files.add(u)
+                    urls2.append(u)
+                    starts2.append(s)
+                    ends2.append(e)
+                    paths2.append(p)
+            for u, s, e, p in zip(urls, starts, ends, paths):
+                if s is not None and u not in whole_files:
+                    urls2.append(u)
+                    starts2.append(s)
+                    ends2.append(e)
+                    paths2.append(p)
+            new_paths, new_starts, new_ends = merge_offset_ranges(
+                list(urls2),
+                list(starts2),
+                list(ends2),
+                sort=True,
+                max_gap=self.max_gap,
+                max_block=self.max_block,
+            )
+            bytes_out = fs.cat_ranges(new_paths, new_starts, new_ends)
+            if len(urls2) == len(bytes_out):
+                # we didn't do any merging
+                for p, b in zip(paths2, bytes_out):
+                    out[p] = b
+            else:
+                # unbundle from merged bytes - simple approach
+                for u, s, e, p in zip(urls, starts, ends, paths):
+                    if p in out:
+                        continue  # was bytes, already handled
+                    for np, ns, ne, b in zip(
+                        new_paths, new_starts, new_ends, bytes_out
+                    ):
+                        if np == u and (ns is None or ne is None):
+                            out[p] = b[s:e]
+                        elif np == u and s >= ns and e <= ne:
+                            out[p] = b[s - ns : (e - ne) or None]
+
+        if len(out) == 1 and isinstance(path, str) and "*" not in path:
+            return _first(out)
+        return out
 
     def _process_dataframe(self):
         self._process_templates(self.templates)
 
         @lru_cache(1000)
         def _render_jinja(url):
-            import jinja2
-
             if "{{" in url:
                 if self.simple_templates:
                     return (
@@ -273,13 +377,14 @@ class ReferenceFileSystem(AsyncFileSystem):
                         .format(**self.templates)
                     )
 
+                import jinja2
+
                 return jinja2.Template(url).render(**self.templates)
 
             return url
 
         if self.templates:
             self.df["url"] = self.df["url"].map(_render_jinja)
-        self._dircache_from_items()
 
     def _process_references(self, references, template_overrides=None):
         if isinstance(references, (str, bytes)):
@@ -293,7 +398,6 @@ class ReferenceFileSystem(AsyncFileSystem):
             raise ValueError(f"Unknown reference spec version: {vers}")
         # TODO: we make dircache by iterating over all entries, but for Spec >= 1,
         #  can replace with programmatic. Is it even needed for mapper interface?
-        self._dircache_from_items()
 
     def _process_references0(self, references):
         """Make reference dict for Spec Version 0"""
@@ -320,7 +424,7 @@ class ReferenceFileSystem(AsyncFileSystem):
                 if v.startswith("base64:"):
                     self.references[k] = base64.b64decode(v[7:])
                 self.references[k] = v
-            else:
+            elif self.templates:
                 u = v[0]
                 if "{{" in u:
                     if self.simple_templates:
@@ -332,16 +436,19 @@ class ReferenceFileSystem(AsyncFileSystem):
                     else:
                         u = _render_jinja(u)
                 self.references[k] = [u] if len(v) == 1 else [u, v[1], v[2]]
+            else:
+                self.references[k] = v
         self.references.update(self._process_gen(references.get("gen", [])))
 
     def _process_templates(self, tmp):
-        import jinja2
 
         self.templates = {}
         if self.template_overrides is not None:
             tmp.update(self.template_overrides)
         for k, v in tmp.items():
             if "{{" in v:
+                import jinja2
+
                 self.templates[k] = lambda temp=v, **kwargs: jinja2.Template(
                     temp
                 ).render(**kwargs)
@@ -349,7 +456,6 @@ class ReferenceFileSystem(AsyncFileSystem):
                 self.templates[k] = v
 
     def _process_gen(self, gens):
-        import jinja2
 
         out = {}
         for gen in gens:
@@ -364,6 +470,8 @@ class ReferenceFileSystem(AsyncFileSystem):
                 for values in itertools.product(*dimension.values())
             )
             for pr in products:
+                import jinja2
+
                 key = jinja2.Template(gen["key"]).render(**pr, **self.templates)
                 url = jinja2.Template(gen["url"]).render(**pr, **self.templates)
                 if ("offset" in gen) and ("length" in gen):
@@ -422,46 +530,105 @@ class ReferenceFileSystem(AsyncFileSystem):
 
     def ls(self, path, detail=True, **kwargs):
         path = self._strip_protocol(path)
+        if not self.dircache:
+            self._dircache_from_items()
         out = self._ls_from_cache(path)
         if out is None:
-            raise FileNotFoundError
+            raise FileNotFoundError(path)
         if detail:
             return out
         return [o["name"] for o in out]
 
     def exists(self, path, **kwargs):  # overwrite auto-sync version
-        try:
-            return self._ls_from_cache(path) is not None
-        except FileNotFoundError:
-            return False
+        return self.isdir(path) or self.isfile(path)
 
     def isdir(self, path):  # overwrite auto-sync version
-        return self.exists(path) and self.info(path)["type"] == "directory"
+        if self.dircache:
+            return path in self.dircache
+        else:
+            # this may be faster than building dircache for single calls, but
+            # by looping will be slow for many calls; could cache it?
+            return any(_.startswith(f"{path}/") for _ in self.references)
 
     def isfile(self, path):  # overwrite auto-sync version
-        return self.exists(path) and self.info(path)["type"] == "file"
+        return path in self.references
 
     async def _ls(self, path, detail=True, **kwargs):  # calls fast sync code
         return self.ls(path, detail, **kwargs)
 
-    def find(self, path, maxdepth=None, withdirs=False, **kwargs):
+    def find(self, path, maxdepth=None, withdirs=False, detail=False, **kwargs):
+        # TODO: details
         if withdirs:
-            return super().find(path, maxdepth=maxdepth, withdirs=withdirs, **kwargs)
+            return super().find(
+                path, maxdepth=maxdepth, withdirs=withdirs, detail=detail, **kwargs
+            )
         if path:
             path = self._strip_protocol(path)
-            return sorted(k for k in self.references if k.startswith(path))
-        return sorted(self.references)
+            r = sorted(k for k in self.references if k.startswith(path))
+        else:
+            r = sorted(self.references)
+        if detail:
+            if not self.dircache:
+                self._dircache_from_items()
+            return {k: self._ls_from_cache(k) for k in r}
+        else:
+            return r
 
     def info(self, path, **kwargs):
-        out = self.ls(path, True)
-        out0 = [o for o in out if o["name"] == path]
-        if not out0:
-            return {"name": path, "type": "directory", "size": 0}
+        if path in self.references:
+            out = self.references[path]
+            if isinstance(out, (str, bytes)):
+                # decode base64 here
+                return {"name": path, "type": "file", "size": len(out)}
+            elif len(out) > 1:
+                return {"name": path, "type": "file", "size": out[2]}
+            else:
+                out0 = [{"name": path, "type": "file", "size": None}]
+        else:
+            out = self.ls(path, True)
+            out0 = [o for o in out if o["name"] == path]
+            if not out0:
+                return {"name": path, "type": "directory", "size": 0}
+        if out0[0]["size"] is None:
+            # if this is a whole remote file, update size using remote FS
+            prot, _ = split_protocol(self.references[path][0])
+            out0[0]["size"] = self.fss[prot].size(self.references[path][0])
         return out0[0]
 
     async def _info(self, path, **kwargs):  # calls fast sync code
         return self.info(path)
 
+    async def _rm_file(self, path, **kwargs):
+        self.references.pop(
+            path, None
+        )  # ignores FileNotFound, just as well for directories
+        self.dircache.clear()  # this is a bit heavy handed
+
+    async def _pipe_file(self, path, data):
+        # can be str or bytes
+        self.references[path] = data
+        self.dircache.clear()  # this is a bit heavy handed
+
+    async def _put_file(self, lpath, rpath):
+        # puts binary
+        with open(lpath, "rb") as f:
+            self.references[rpath] = f.read()
+        self.dircache.clear()  # this is a bit heavy handed
+
+    def save_json(self, url, **storage_options):
+        """Write modified references into new location"""
+        out = {}
+        for k, v in self.references.items():
+            if isinstance(v, bytes):
+                try:
+                    out[k] = v.decode("ascii")
+                except UnicodeDecodeError:
+                    out[k] = (b"base64:" + base64.b64encode(v)).decode()
+            else:
+                out[k] = v
+        with fsspec.open(url, "wb", **storage_options) as f:
+            f.write(json.dumps({"version": 1, "refs": out}).encode())
+
 
 def _unmodel_hdf5(references):
     """Special JSON format from HDF5 prototype"""
