@@ -1,3 +1,4 @@
+import os
 import shlex
 import subprocess
 import time
@@ -93,6 +94,27 @@ def test_with_url(protocol, ssh):
         assert f.read() == b"hello"
 
 
+@pytest.mark.parametrize("protocol", ["sftp", "ssh"])
+def test_get_dir(protocol, ssh, root_path):
+    f = fsspec.filesystem(protocol, **ssh)
+    f.mkdirs(root_path + "deeper", exist_ok=True)
+    f.touch(root_path + "deeper/afile")
+    f.get(root_path, ".", recursive=True)
+
+    assert os.path.isdir("./deeper")
+    assert os.path.isfile("./deeper/afile")
+
+    f.get(
+        protocol + "://{username}:{password}@{host}:{port}"
+        "{root_path}".format(root_path=root_path, **ssh),
+        "./test2",
+        recursive=True,
+    )
+
+    assert os.path.isdir("./test2/deeper")
+    assert os.path.isfile("./test2/deeper/afile")
+
+
 @pytest.fixture(scope="module")
 def netloc(ssh):
     username = ssh.get("username")
