@@ -1,4 +1,6 @@
-from fsspec.callbacks import Callback
+import pytest
+
+from fsspec.callbacks import Callback, TqdmCallback
 
 
 def test_callbacks():
@@ -36,3 +38,19 @@ def test_callbacks_wrap():
         ...
 
     assert events == [1] * 10
+
+
+@pytest.mark.parametrize("tqdm_kwargs", [{}, {"desc": "A custom desc"}])
+def test_tqdm_callback(tqdm_kwargs, mocker):
+
+    callback = TqdmCallback(tqdm_kwargs=tqdm_kwargs)
+    mocker.patch.object(callback, "_tqdm")
+    callback.set_size(10)
+    for _ in callback.wrap(range(10)):
+        ...
+
+    assert callback.tqdm.update.call_count == 10
+    if not tqdm_kwargs:
+        callback._tqdm.tqdm.assert_called_with(total=10)
+    else:
+        callback._tqdm.tqdm.assert_called_with(total=10, **tqdm_kwargs)
