@@ -6,6 +6,7 @@ import pytest
 from fsspec.registry import (
     ReadOnlyError,
     _registry,
+    filesystem,
     get_filesystem_class,
     known_implementations,
     register_implementation,
@@ -104,3 +105,18 @@ def test_entry_points_registered_on_import(clear_registry, clean_imports):
 
         get_filesystem_class("test")
         assert "test" in registry
+
+
+def test_filesystem_warning_arrow_hdfs_deprecated(clear_registry, clean_imports):
+    mock_ep = create_autospec(EntryPoint, module="fsspec.spec.AbstractFileSystem")
+    mock_ep.name = "arrow_hdfs"  # this can't be set in the constructor...
+    mock_ep.value = "fsspec.spec.AbstractFileSystem"
+    if sys.version_info < (3, 8):
+        import_location = "importlib_metadata.entry_points"
+    else:
+        import_location = "importlib.metadata.entry_points"
+    with patch(import_location, return_value={"fsspec.specs": [mock_ep]}):
+        import fsspec  # noqa
+
+        with pytest.warns(DeprecationWarning):
+            filesystem("arrow_hdfs")
