@@ -248,6 +248,7 @@ def test_cache():
     fs2 = DummyTestFS()
     assert fs is fs2
 
+    assert DummyTestFS.current() is fs
     assert len(fs._cache) == 1
     del fs2
     assert len(fs._cache) == 1
@@ -260,6 +261,17 @@ def test_cache():
     assert len(DummyTestFS._cache) == 0
 
 
+def test_current():
+    fs = DummyTestFS()
+    fs2 = DummyTestFS(arg=1)
+
+    assert fs is not fs2
+    assert DummyTestFS.current() is fs2
+
+    DummyTestFS()
+    assert DummyTestFS.current() is fs
+
+
 def test_alias():
     with pytest.warns(FutureWarning, match="add_aliases"):
         DummyTestFS(add_aliases=True)
@@ -554,12 +566,9 @@ def test_dummy_callbacks_files_branched(tmpdir):
 
         base_keys = zip(make_path_posix(lpaths), make_path_posix(rpaths))
         assert set(callback.events.keys()) == {("top-level",), *base_keys}
-        assert (
-            callback.events[
-                "top-level",
-            ]
-            == imitate_transfer(10, 10, file=False)
-        )
+        assert callback.events[
+            "top-level",
+        ] == imitate_transfer(10, 10, file=False)
 
         for key in base_keys:
             assert callback.events[key] == imitate_transfer(50, 5)
