@@ -9,8 +9,7 @@ from functools import partial
 from hashlib import md5
 from urllib.parse import urlsplit
 
-DEFAULT_BLOCK_SIZE = 5 * 2 ** 20
-PY36 = sys.version_info < (3, 7)
+DEFAULT_BLOCK_SIZE = 5 * 2**20
 
 
 def infer_storage_options(urlpath, inherit_storage_options=None):
@@ -242,14 +241,14 @@ def read_block(f, offset, length, delimiter=None, split_before=False):
     """
     if delimiter:
         f.seek(offset)
-        found_start_delim = seek_delimiter(f, delimiter, 2 ** 16)
+        found_start_delim = seek_delimiter(f, delimiter, 2**16)
         if length is None:
             return f.read()
         start = f.tell()
         length -= start - offset
 
         f.seek(start + length)
-        found_end_delim = seek_delimiter(f, delimiter, 2 ** 16)
+        found_end_delim = seek_delimiter(f, delimiter, 2**16)
         end = f.tell()
 
         # Adjust split location to before delimiter iff seek found the
@@ -301,8 +300,8 @@ def stringify_path(filepath):
 
     Notes
     -----
-    Objects supporting the fspath protocol (Python 3.6+) are coerced
-    according to its __fspath__ method.
+    Objects supporting the fspath protocol are coerced according to its
+    __fspath__ method.
 
     For backwards compatibility with older Python version, pathlib.Path
     objects are specially coerced.
@@ -385,6 +384,13 @@ def is_exception(obj):
     return isinstance(obj, BaseException)
 
 
+def isfilelike(f):
+    for attr in ["read", "close", "tell"]:
+        if not hasattr(f, attr):
+            return False
+    return True
+
+
 def get_protocol(url):
     parts = re.split(r"(\:\:|\://)", url, 1)
     if len(parts) > 1:
@@ -456,14 +462,7 @@ def setup_logging(logger=None, logger_name=None, level="DEBUG", clear=True):
 
 
 def _unstrip_protocol(name, fs):
-    if isinstance(fs.protocol, str):
-        if name.startswith(fs.protocol):
-            return name
-        return fs.protocol + "://" + name
-    else:
-        if name.startswith(tuple(fs.protocol)):
-            return name
-        return fs.protocol[0] + "://" + name
+    return fs.unstrip_protocol(name)
 
 
 def mirror_from(origin_name, methods):
@@ -497,7 +496,6 @@ def merge_offset_ranges(paths, starts, ends, max_gap=0, max_block=None, sort=Tru
     order. If the user can guarantee that the inputs are already
     sorted, passing `sort=False` will skip the re-ordering.
     """
-
     # Check input
     if not isinstance(paths, list):
         raise TypeError
@@ -541,3 +539,12 @@ def merge_offset_ranges(paths, starts, ends, max_gap=0, max_block=None, sort=Tru
 
     # `paths` is empty. Just return input lists
     return paths, starts, ends
+
+
+def file_size(filelike):
+    """Find length of any open read-mode file-like"""
+    pos = filelike.tell()
+    try:
+        return filelike.seek(0, 2)
+    finally:
+        filelike.seek(pos)
